"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _Protocol = require("./Protocol");

var _Protocol2 = _interopRequireDefault(_Protocol);

var _crystalEventEmitter = require("crystal-event-emitter");

var _crystalEventEmitter2 = _interopRequireDefault(_crystalEventEmitter);

var _addDefaults = require("./addDefaults");

var _addDefaults2 = _interopRequireDefault(_addDefaults);

var _proxify = require("./proxify");

var _proxify2 = _interopRequireDefault(_proxify);

var _codes = require("./codes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Client extends _crystalEventEmitter2.default {
	constructor(...args) {
		super({
			inferListeners: true
		});
		this.ws = null;
		this.network = null;
		this.reconnecting = false;
		this.options = {};
		if (!args.length) {
			throw new Error("No arguments provided");
		}
		const [url] = args;
		this.url = url;
		if (args.length === 2) {
			/* User wants to specify options or protocols */
			const [, arg2] = args;
			if (typeof arg2 === "object") {
				/* He wants to specify options */
				this.options = arg2;
				this.protocols = null;
			} else {
				/* He wants to specify protocols */
				this.protocols = arg2;
			}
		}
		if (args.length === 3) {
			/* User wants to specify options and protocols */
			const [, protocols, options = {}] = args;
			this.protocols = protocols;
			this.options = options;
		}
		this.options = (0, _addDefaults2.default)(this.options, {
			engine: global.WebSocket,
			autoReconnect: true,
			reconnectionMinimum: 200,
			reconnectionFactor: 1.15,
			binaryType: "arraybuffer"
		});
		if (!this.options.engine) {
			throw new Error("No WebSocket client implementation found. If your environment doesn't natively support WebSockets, please provide the client class to use with the `engine` option.");
		}
		this.proxy = (0, _proxify2.default)(this, (0, _assign2.default)({}, this.options, {
			bind: true
		}));
		return this.proxy;
	}
	clear(e) {
		this.emit("close", e);
		this.ws = null;
		this.network = null;
	}
	open() {
		var _this = this;

		return new _promise2.default((() => {
			var _ref = (0, _asyncToGenerator3.default)(function* (resolve, reject) {
				if (_this.ws) {
					yield _this.close();
				}
				_this.ws = new _this.options.engine(_this.url, _this.protocols, _this.options.engineOptions);
				_this.ws.binaryType = _this.options.binaryType;
				_this.ws.onopen = function (e) {
					_this.emit("open", e);
					resolve(_this.proxy);
				};
				_this.ws.onerror = function (e) {
					_this.emit("error", e);
					reject(e);
					// this.close();
				};
				/* Closed dirtily */
				_this.ws.onclose = function (e) {
					const { code } = e;
					_this.clear(e);
					if (code !== _codes.CLOSE_NORMAL) {
						if (_this.options.autoReconnect && !_this.reconnecting) {
							_this.reconnect();
						}
					}
				};
				_this.ws.onmessage = function (e) {
					return _this.network.read(e.data);
				};
				_this.network = new _Protocol2.default(_this.ws, _this.options);
				_this.network.on("*", function (...args) {
					_this.emit(...args);
				});
			});

			return function (_x, _x2) {
				return _ref.apply(this, arguments);
			};
		})());
	}
	reconnect(delay) {
		var _this2 = this;

		return (0, _asyncToGenerator3.default)(function* () {
			_this2.reconnecting = true;
			const timeout = delay || _this2.options.reconnectionMinimum;
			try {
				yield _this2.open();
				_this2.reconnecting = false;
				_this2.emit("reconnect");
			} catch (e) {
				setTimeout(function () {
					_this2.reconnect(timeout * _this2.options.reconnectionFactor);
				}, timeout);
			}
		})();
	}
	close() {
		return new _promise2.default((resolve, reject) => {
			if (this.ws) {
				/* Closed cleanly */
				this.ws.onclose = e => {
					this.clear(e);
					resolve(this.proxy);
				};
				this.ws.close();
			} else {
				reject(new Error("WebSocket hasn't been initialized"));
			}
		});
	}
	send(message) {
		var _this3 = this;

		return (0, _asyncToGenerator3.default)(function* () {
			if (!_this3.network) {
				throw new Error(`Attempted to send command "${message.instruction.command}" without a connection being established`);
			}
			return _this3.network.send(message);
		})();
	}
}
exports.default = Client;