"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _Instruction = require("./Instruction");

var _v = require("uuid/v4");

var _v2 = _interopRequireDefault(_v);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
* Messages are Instruction containers.
* They're the essential payload that will be encoded later.
*/
class Message {
	constructor(instruction, options, id = (0, _v2.default)()) {
		this.instruction = instruction;
		this.options = options;
		this.id = id;
	}
	makeReply(...args) {
		let nextInstruction;
		const { instruction, options, id } = this;
		if (instruction instanceof _Instruction.SYN) {
			nextInstruction = _Instruction.ACK;
		} else if (instruction instanceof _Instruction.ACK) {
			nextInstruction = _Instruction.SYN_ACK;
		} else {
			throw new Error("Invalid attempt to reply to a reply");
		}
		return new Message(new nextInstruction(instruction.command, ...args), options, id);
	}
	encode() {
		return this.options.encode({
			id: this.id,
			instruction: this.instruction
		});
	}
	static from(encoded, options) {
		let preprocessed = encoded;
		if (encoded instanceof ArrayBuffer) {
			preprocessed = new Uint8Array(encoded);
		}
		const object = options.decode(preprocessed);
		const { type, command, args } = object.instruction;
		const [constructor] = [_Instruction.SYN, _Instruction.ACK, _Instruction.SYN_ACK].filter(c => c.type === type);
		const instruction = new constructor(command, ...args);
		return new this(instruction, options, object.id);
	}
}
exports.default = Message;