"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.SYN_ACK = exports.ACK = exports.SYN = undefined;

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _Message = require("./Message");

var _Message2 = _interopRequireDefault(_Message);

var _Instruction = require("./Instruction");

var _crystalEventEmitter = require("crystal-event-emitter");

var _crystalEventEmitter2 = _interopRequireDefault(_crystalEventEmitter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.SYN = _Instruction.SYN;
exports.ACK = _Instruction.ACK;
exports.SYN_ACK = _Instruction.SYN_ACK;
class Protocol extends _crystalEventEmitter2.default {
	constructor(ws, options) {
		super();
		this.internal = new _crystalEventEmitter2.default();
		this.ws = ws;
		this.options = options;
	}
	read(encoded) {
		/* "Reading" means: Return the decoded message, but don't reply. This is the user's responsibility; instead, append a `reply` function so that the user can pass in arguments in response. */
		const message = _Message2.default.from(encoded, this.options);
		/* Every event should have a reference to the client responsible for it */
		message.client = this;
		message.reply = (...args) => {
			const replyMessage = message.makeReply(...args);
			return this.send(replyMessage);
		};
		const { instruction, id } = message;
		if (instruction instanceof _Instruction.SYN) {
			const { command, args } = instruction;
			/* Specific event */
			this.emit(command, message, ...args);
			/* General event */
			this.emit("message", command, message, ...args);
		}
		if (instruction instanceof _Instruction.ACK || instruction instanceof _Instruction.SYN_ACK) {
			/* Fire an internal event so that a previous `send` action can be completed */
			this.internal.emit(id, message);
		}
		return message;
	}
	send(message) {
		return new _promise2.default(resolve => {
			const { id } = message;
			this.internal.once(id, reply => {
				const { instruction } = reply;
				const { args } = instruction;
				resolve([reply, ...args]);
				/* TODO: Handle rejection/listener removal based on timeout */
			});
			this.ws.send(message.encode());
		});
	}
}
exports.default = Protocol;