"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _map = require("babel-runtime/core-js/map");

var _map2 = _interopRequireDefault(_map);

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _set = require("babel-runtime/core-js/set");

var _set2 = _interopRequireDefault(_set);

var _ws = require("ws");

var _crystalEventEmitter = require("crystal-event-emitter");

var _crystalEventEmitter2 = _interopRequireDefault(_crystalEventEmitter);

var _Protocol = require("./Protocol");

var _Protocol2 = _interopRequireDefault(_Protocol);

var _addDefaults = require("./addDefaults");

var _addDefaults2 = _interopRequireDefault(_addDefaults);

var _proxify = require("./proxify");

var _proxify2 = _interopRequireDefault(_proxify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Server extends _crystalEventEmitter2.default {
	constructor(options = {}) {
		super({
			inferListeners: true
		});
		this.clients = new _set2.default();
		this.options = (0, _addDefaults2.default)(options, {
			engine: _ws.Server
		});
	}
	broadcast(message) {
		var _this = this;

		return (0, _asyncToGenerator3.default)(function* () {
			const replies = new _map2.default();
			for (const client of _this.clients) {
				const clientMessage = yield client.send(message);
				replies.set(client, clientMessage);
			}
			return replies;
		})();
	}
	clear() {
		this.clients.clear();
		this.wss = null;
	}
	open() {
		return new _promise2.default(resolve => {
			this.wss = new this.options.engine(this.options.engineOptions);
			this.wss.on("listening", () => {
				resolve(this);
			});
			this.wss.on("connection", (ws, request) => {
				const client = (0, _proxify2.default)(new _Protocol2.default(ws, this.options), (0, _assign2.default)({}, this.options));
				client.request = request;
				/* Actual clients can handle `close` themselves (due to auto-reconnection). `Protocol`s can't. */
				client.close = () => new _promise2.default(resolve => {
					ws.on("close", () => {
						resolve();
					});
					ws.close();
				});
				/* Take note of the client so that the server can reference it */
				this.clients.add(client);
				this.emit("connection", client);
				/* The endpoint should pass messages through the protocol */
				ws.on("message", encoded => client.read(encoded));
				ws.on("close", e => {
					this.clients.delete(client);
					client.emit("close");
					this.emit("clientClose", client, e);
				});
				/* The protocol emissions are forwarded */
				client.on("*", (event, ...args) => {
					/* Don't forward the `close` event; server-side clients will receive `clientClose` */
					if (event !== "close") {
						this.emit(...args);
					}
				});
			});
		});
	}
	close() {
		return new _promise2.default((resolve, reject) => {
			this.wss.close(error => {
				if (error) {
					reject(error);
				} else {
					resolve(this);
					this.clear();
				}
			});
		});
	}
}
exports.default = Server;